#!/usr/bin/env python
# -*- coding: utf-8 -*-
# pylint: disable=invalid-name, too-many-arguments, broad-except
# pylint: disable=missing-docstring, line-too-long
# pylint: anomalous-backslash-in-string


def image_preprocessing(img_origin_path, img_target_path):
    """
        图片降噪处理
    :param img_origin_path:
        ［string］原始图片位置
    :param img_target_path:
        ［string］图片保存位置
    """
    try:
        from PIL import Image
    except:
        print '请安装pillow库'
        return

    img = Image.open(img_origin_path)  # PIL库加载图片
    # img = img.convert('RGBA')  # 转换为RGBA
    pix = img.load()  # 读取为像素
    for x in range(img.size[0]):  # 处理上下黑边框
        pix[x, 0] = pix[x, img.size[1] - 1] = (255, 255, 255, 255)
    for y in range(img.size[1]):  # 处理左右黑边框
        pix[0, y] = pix[img.size[0] - 1, y] = (255, 255, 255, 255)
    for y in range(img.size[1]):  # 二值化处理，这个阈值为R=95，G=95，B=95
        for x in range(img.size[0]):
            if pix[x, y][0] < 95 or pix[x, y][1] < 95 or pix[x, y][2] < 95:
                pix[x, y] = (0, 0, 0, 255)
            else:
                pix[x, y] = (255, 255, 255, 255)
    img.save(img_target_path)


def vcode_recognition(img_path):
    """
        验证码识别
    :param img_path:
        ［string］原始图片位置
    """
    try:
        from PIL import Image
    except:
        print '请安装pillow库'
        return

    try:
        from pytesseract import pytesseract as fp
    except:
        print "请安装pytesseract库"
        return

    image_preprocessing(img_path, img_path)

    im = Image.open(img_path)
    text = pt.image_to_string(im, lang="eng", config="-psm 3")
    text = text.strip()
    text = text.upper()
    return filter(str.isalnum, text.encode('utf-8'))


class ValidateCode(object):

    _letter_cases = "abcdefghjkmnpqrstuvwxy"  # 小写字母，去除可能干扰的i，l，o，z
    _upper_cases = _letter_cases.upper()  # 大写字母
    _numbers = ''.join(map(str, range(3, 10)))  # 数字
    init_chars = ''.join((_letter_cases, _upper_cases, _numbers))

    def __init__(self):
        pass

    def create(self,
               size=(120, 30),
               chars=init_chars,
               img_type="GIF",
               mode="RGB",
               bg_color=(255, 255, 255),
               fg_color=(0, 0, 255),
               font_size=18,
               font_type="Times New Roman.ttf",
               length=4,
               draw_lines=True,
               n_line=(1, 2),
               draw_points=True,
               point_chance=2):
        """
        @todo: 生成验证码图片
        @param size: 图片的大小，格式（宽，高），默认为(120, 30)
        @param chars: 允许的字符集合，格式字符串
        @param img_type: 图片保存的格式，默认为GIF，可选的为GIF，JPEG，TIFF，PNG
        @param mode: 图片模式，默认为RGB
        @param bg_color: 背景颜色，默认为白色
        @param fg_color: 前景色，验证码字符颜色，默认为蓝色#0000FF
        @param font_size: 验证码字体大小
        @param font_type: 验证码字体，默认为 ae_AlArabiya.ttf
        @param length: 验证码字符个数
        @param draw_lines: 是否划干扰线
        @param n_line: 干扰线的条数范围，格式元组，默认为(1, 2)，只有draw_lines为True时有效
        @param draw_points: 是否画干扰点
        @param point_chance: 干扰点出现的概率，大小范围[0, 100]
        @return: [0]: PIL Image实例
        @return: [1]: 验证码图片中的字符串
        """
        import random
        try:
            from PIL import Image, ImageDraw, ImageFont, ImageFilter
        except:
            print '请安装pillow库'
            return

        width, height = size  # 宽， 高
        img = Image.new(mode, size, bg_color)  # 创建图形
        draw = ImageDraw.Draw(img)  # 创建画笔

        def get_chars():
            """ 生成给定长度的字符串，返回列表格式 """
            return random.sample(chars, length)

        def create_lines():
            """ 绘制干扰线 """
            line_num = random.randint(*n_line)  # 干扰线条数

            for i in range(line_num):
                # 起始点
                begin = (random.randint(0, size[0]), random.randint(
                    0, size[1]))
                # 结束点
                end = (random.randint(0, size[0]), random.randint(0, size[1]))
                draw.line([begin, end], fill=(0, 0, 0))

        def create_points():
            """ 绘制干扰点 """
            chance = min(100, max(0, int(point_chance)))  # 大小限制在[0, 100]

            for w in xrange(width):
                for h in xrange(height):
                    tmp = random.randint(0, 100)
                    if tmp > 100 - chance:
                        draw.point((w, h), fill=(0, 0, 0))

        def create_strs():
            """ 绘制验证码字符 """
            c_chars = get_chars()
            strs = ' %s ' % ' '.join(c_chars)  # 每个字符前后以空格隔开

            font = ImageFont.truetype(font_type, font_size)
            font_width, font_height = font.getsize(strs)

            draw.text(
                ((width - font_width) / 3, (height - font_height) / 3),
                strs,
                font=font,
                fill=fg_color)

            return ''.join(c_chars)

        if draw_lines:
            create_lines()
        if draw_points:
            create_points()
        strs = create_strs()

        # 图形扭曲参数
        params = [
            1 - float(random.randint(1, 2)) / 100, 0, 0, 0,
            1 - float(random.randint(1, 10)) / 100,
            float(random.randint(1, 2)) / 500, 0.001,
            float(random.randint(1, 2)) / 500
        ]
        img = img.transform(size, Image.PERSPECTIVE, params)  # 创建扭曲

        img = img.filter(ImageFilter.EDGE_ENHANCE_MORE)  # 滤镜，边界加强（阈值更大）

        return img, strs


def decode_image_from_base64(b64_str):
    """ decoding from base64 string to image """
    import base64
    import cStringIO
    try:
        from PIL import Image
    except:
        print '请安装pillow库'
        return

    img_data = base64.b64decode(b64_str)
    img = Image.open(cStringIO.StringIO(img_data))
    return img


def encode_base64_from_image(img):
    """ encoding from image to base64 string """
    import base64
    import cStringIO
    buffer = cStringIO.StringIO()
    img.save(buffer, format="JPEG")
    img_str = base64.b64encode(buffer.getvalue())
    return img_str
