#!/usr/bin/env python
# -*- coding: utf-8 -*-
# pylint: disable=invalid-name, too-many-arguments, broad-except,
# pylint: disable=missing-docstring, line-too-long
# pylint: anomalous-backslash-in-string

import time
import multiprocessing
import requests
from prettytable import PrettyTable

from printPatch import ColorString
from utils import get_latency_str_for_millisecond

#######################################################
# GLOBAL variable
RES_QUEUE = []
ERROR_C = 0
proc_current_pos = 0
proc_current_persent = 0
proc_t = 0


def post_unit(session, requests_url):
    """
        请求模块
    :param session:
    :param requests_url:
    """
    if isinstance(requests_url, str) is False:
        [session, requests_url] = [requests_url, session]

    try:
        start_t = time.time()
        res = session.get(requests_url, verify=True)
        proc_t = time.time() - start_t
        return [res.status_code, proc_t]
    except Exception as error:
        print(error)
        return [0, 0]


class testing(object):
    def __init__(self,
                 requests_url,
                 requests_c=100,
                 thread_c=multiprocessing.cpu_count() * 2 + 1):
        """
            init
        :param requests_url:
            [string] 测试url
        :param requests_c:
            [int] 测试次数
        :param thread_c:
            [int] 请求线程
        """
        self.session = requests.session()
        self.requests_url = requests_url
        self.requests_c = requests_c
        self.thread_c = thread_c
        self.multi_pool = None

    def thread_callback(self, res):
        """ callback """
        global ERROR_C
        global proc_current_pos
        global proc_current_persent

        proc_current_pos += 1
        persent = proc_current_pos * 100 / self.requests_c
        if persent % 5 == 0 and persent > proc_current_persent:
            proc_current_persent = persent
            print(ColorString.format('proc...%s%%' % proc_current_persent,
                                     ColorString.YELLOW))

        if res[0] != 200:
            ERROR_C += 1
        RES_QUEUE.append(res[1])

    def run(self):
        """ run """
        global proc_t
        self.init_global_status()
        print(ColorString.format('[* testing start *]', ColorString.GREEN))
        start_t = time.time()
        per_c = 1000 if self.requests_c > 1000 else self.requests_c

        for i in range(self.requests_c / per_c):
            self.multi_pool = multiprocessing.Pool(processes=self.thread_c)
            cur_per_c = self.requests_c - i * per_c + per_c if i * per_c > self.requests_c else per_c
            for j in range(cur_per_c):
                self.multi_pool.apply_async(
                    post_unit,
                    args={self.session, self.requests_url},
                    callback=self.thread_callback)
            self.multi_pool.close()
            self.multi_pool.join()
        proc_t = (time.time() - start_t)
        print(ColorString.format('[* testing end *]', ColorString.GREEN))
        self.results()

    @staticmethod
    def init_global_status():
        """ init global status variable """
        global RES_QUEUE, ERROR_C, proc_current_pos, proc_current_persent, proc_t
        RES_QUEUE = []
        ERROR_C = 0
        proc_current_pos = 0
        proc_current_persent = 0
        proc_t = 0

    def results(self):
        """ testing status result """

        print(ColorString.format('*' * 50, ColorString.CYAN))
        print(ColorString.format("线程数：%s" % self.thread_c, ColorString.CYAN))
        print(
            ColorString.format("总请求量：%s" % proc_current_pos, ColorString.CYAN))
        print(ColorString.format("请求失败：%s (%d%%)" % (ERROR_C, (
            ERROR_C * 100 / proc_current_pos)), ColorString.RED))
        print(ColorString.format("耗时：%0.2fs" % proc_t, ColorString.CYAN))
        print(ColorString.format('*' * 50, ColorString.CYAN))

        res_dic = dict()
        for i in RES_QUEUE:
            latency = get_latency_str_for_millisecond(i * 1000)
            if res_dic.get(latency, None) is None:
                res_dic[latency] = 0
            res_dic[latency] += 1

        status_table = PrettyTable(["timeout", "count"])
        status_table.padding_width = 1
        for i in res_dic.keys():
            status_table.add_row([i, res_dic[i]])
        print(ColorString.format(status_table, ColorString.GREEN))


if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser(description="some information here")
    parser.add_argument(
        "-url",
        "--request_url",
        type=str,
        required=True,
        help="test target url")
    parser.add_argument("-c", "--request_count", type=int, help="test count")
    args = parser.parse_args()
    req_url = args.request_url

    req_c = args.request_count if args.request_count else 100

    testing(req_url, req_c).run()
